#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdbool.h>
#include <errno.h>
#include "mandelbrotfunction.h"


bool string_to_int(char* s, int* val) {
   char* endptr;
   errno = 0; /* reset errno */
   int i = strtol(s, &endptr, 10);
   if (endptr == s) return false; /* conversion failed */
   if (*endptr) return false; /* unexpected characters behind the integer */
   if (errno) return false; /* out of range */
   *val = i;
   return true;
}

bool string_to_double(char* s, double* val) {
   char* endptr;
   errno = 0; /* reset errno */
   double x = strtod(s, &endptr);
   if (endptr == s) return false; /* conversion failed */
   if (*endptr) return false; /* unexpected characters behind the double */
   if (errno) return false; /* out of range */
   *val = x;
   return true;
}

void parse_double_par(double* par, char** argv) {
    if (!*argv || !string_to_double(*argv, par)) {
        printf("Error: Expected valid double after option %s\n", *--argv); 
        exit(1);
    }
}

void parse_int_par(int* par, char** argv) {
    if (!*argv || !string_to_int(*argv, par)) {
        printf("Error: Expected valid int after option %s\n", *--argv); 
        exit(1);
    }
}

int get_num_iter(double c_re, double c_im, int max_iter) {
    double z_n_re = 0;
    double z_n_im = 0;
    for (int it = 1; it < max_iter; ++it) {
        // compute z_{n+1} as z_n ^ 2 + c  
        double z_np1_re = z_n_re * z_n_re - z_n_im * z_n_im + c_re;
        double z_np1_im = 2 * z_n_re * z_n_im + c_im;
        // if |z_{n+1}| > 2, (z_n) diverges and c is not in set.
        if (z_np1_re * z_np1_re + z_np1_im * z_np1_im > 4) {
            return it;
        }
        z_n_re = z_np1_re;
        z_n_im = z_np1_im;
    }
    // c is (probably) in set
    return 0;
}
